#include "ws2812.h"

class ws2812_Animation {
public:
    virtual void update();
};

class ws2812_FillWheelDim : public ws2812_Animation {
public:
    ws2812_FillWheelDim(uint32_t *colors, int count, int samples, int brightness)
      : m_colors(colors), m_count(count), m_samples(samples), m_brightness(brightness), m_i(0), m_di(1), m_j(0) {}
    ~ws2812_FillWheelDim() {}
    void update();
private:
    uint32_t *m_colors;
    int m_count;
    int m_samples;
    int m_brightness;
    int m_i;
    int m_di;
    int m_j;
};

void ws2812_FillWheelDim::update()
{
    m_colors[m_i] = ws2812_wheel_dim(m_j * (255 / m_samples), m_brightness);
    m_i += m_di;
    if (m_i >= m_count) {
        m_di = -1;
        --m_i;
    }
    else if (m_i < 0) {
        m_di = 1;
        ++m_i;
    } 
    if (++m_j >= m_samples)
        m_j = 0;    
}

class ws2812_Chase : public ws2812_Animation {
public:
    ws2812_Chase(uint32_t *colors, int count, uint32_t *pattern, int pcount)
      : m_colors(colors), m_count(count), m_pattern(pattern), m_pcount(pcount), m_base(0) {}
    ~ws2812_Chase() {}
    void update();
private:
    uint32_t *m_colors;
    int m_count;
    uint32_t *m_pattern;
    int m_pcount;
    int m_base;
};

void ws2812_Chase::update()
{
    int idx = m_base;                       // start at base
    for (int i = 0; i < m_count; ++i) {     // loop through connected leds
        m_colors[i] = m_pattern[idx];       // update channel color
        if (++idx >= m_pcount)              // past end of list?
            idx = 0;                        // yes, reset
    }
    if (++m_base >= m_pcount)               // update the base for the next time
        m_base = 0;
}

#include <propeller.h>

#define LED_PIN     15
#define LED_COUNT   16
#define LED2_PIN    14
#define LED2_COUNT  8

uint32_t Chakras[] = {
    COLOR_RED,
    COLOR_ORANGE,
    COLOR_YELLOW, 
    COLOR_GREEN,
    COLOR_BLUE,
    COLOR_INDIGO
};
int Chackras_count = sizeof(Chakras) / sizeof(uint32_t);

int main(void)
{
    int cog;

    // 16 LED ring
    uint32_t colors[LED_COUNT];
    ws2812_FillWheelDim wheel(colors, LED_COUNT, LED_COUNT+5, 128);

    // 8 LED strip
    uint32_t colors2[LED2_COUNT];
    ws2812_Chase chase(colors2, LED2_COUNT, Chakras, Chackras_count);

    if ((cog = ws2812_init()) < 0)
        return 1;
        
    for (;;) {
        wheel.update();
        ws2812_execute(LED_PIN, colors, LED_COUNT);
        ws2812_pause(100);
        chase.update();
        ws2812_execute(LED2_PIN, colors2, LED2_COUNT);
        ws2812_pause(100);
    }
           
    cogstop(cog);
    
    return 0;
}
