#include <stdio.h>
#include <propeller.h>
#include "ws2812.h"

#ifndef TRUE
#define TRUE    1
#define FALSE   0
#endif

// driver header structure
typedef struct {
    uint32_t    jmp_inst;
    uint32_t    resettix;
    uint32_t    bit0hi;
    uint32_t    bit0lo;
    uint32_t    bit1hi;
    uint32_t    bit1lo;
    uint32_t    swaprg;
} ws2812_hdr;

// command structure
typedef struct {
    uint8_t pin;
    uint8_t count;
    uint16_t colors;
} ws2812_command;

static volatile ws2812_command command;

int ws2812_init(void)
{
    return ws_init(50, 350, 800, 700, 600, SWAP_RG);
}

int ws2812b_init(void)
{
    return ws_init(50, 350, 900, 900, 350, SWAP_RG);
}

// -- usreset - is reset timing (us)
// -- ns0h is 0-bit high timing (ns)
// -- ns0l is 0-bit low timing (ns)
// -- ns1h is 1-bit high timing (ns)
// -- ns1l is 1-bit low timing (ns)
int ws_init(int usreset, int ns0h, int ns0l, int ns1h, int ns1l, int swaprg)
{
    extern uint32_t binary_ws2812_driver_dat_start[];
    ws2812_hdr *hdr = (ws2812_hdr *)binary_ws2812_driver_dat_start;
    uint32_t ustix;

    if (CLKFREQ != 80000000)            // requires 80MHz clock
        return -1;

    ustix = CLKFREQ / 1000000;          // ticks in 1us

    hdr->resettix = ustix * usreset;
    hdr->bit0hi   = ustix * ns0h / 1000;
    hdr->bit0lo   = ustix * ns0l / 1000;
    hdr->bit1hi   = ustix * ns1h / 1000;
    hdr->bit1lo   = ustix * ns1l / 1000;
    hdr->swaprg   = swaprg;             // swap R and G in color values
    
    *(uint32_t *)&command = 0;
    
    return cognew(hdr, &command);
}

void ws2812_execute(int pin, uint32_t *colors, int count)
{
    ws2812_command cmd;
    cmd.pin = pin;
    cmd.count = count - 1;
    cmd.colors = (uint16_t)(uint32_t)colors;
    while (*(uint32_t *)&command)
        ;
    command = cmd;
}
